#!/bin/ksh
#
# $Id: dumpthreads,v 1.5 1998/07/27 14:42:15 galvis Exp $
#
# (C) COPYRIGHT International Business Machines Corp. 1998
# All Rights Reserved
#
# dumpthreads
#
# DESCRIPTION: The dumpthreads script will list the following information
#              about a running process or core file:
#
#   List of existing threads
#   List of condition variables in the wait state
#   List of mutex variables in the locked state
#   List of stack traces for each existing thread
#
# NOTE: You must have the AIX fileset bos.adt.debug installed on the
#       machine where this script will be run, and you must be root.

usage="
Usage: $0 [-short] process_id_or_name
       $0 [-short] command_path corefile"

echo "
-----------------------------------------------------------------------------
dumpthreads output generated at : `date`
command line : $0 $*
-----------------------------------------------------------------------------
"

case $1 in
-s)
    output="short"
    shift
    ;;
-short)
    output="short"
    shift
    ;;
-long)
    output="long"
    shift
    ;;
*)
    output="long"
    ;;
esac

if [ -z "$1" ]
then
    echo "$usage"
    exit 0
fi

if [ "`whoami`" != "root" ]
then
    echo "You must be root to run this script"
    exit 1
fi

lslpp -L bos.adt.debug 2>/dev/null | grep bos.adt.debug 2>&1 >/dev/null
if [ $? -ne 0 ]
then
    echo "The AIX fileset bos.adt.debug is not installed on this"
    echo "machine.  It must be installed before using dumpthreads."
    exit 1
fi

whence dbx 2>&1 >/dev/null
if [ $? -ne 0 ]
then
    echo "The program \"dbx\" could not be found."
    exit 1
fi

#####################################################################
# ps -ef | awk '{print $2}' | grep -w $pid 2>&1 >/dev/null
# if [ $? -ne 0 ]
# then
#     echo "Process ID \"$pid\" does not exist"
#     exit 1
# fi
#####################################################################

if [ -z "$2" ]
then
    source="process"

    # convert name to number

    foo=`expr $1 + 1 2>/dev/null`
    if [ -z "$foo" ]
    then
      pid=`ps -ef | grep $1 | grep -v grep | grep -v $0 | awk '{print $2;exit}'`
    else
      pid=$1
    fi

    if [ -z "$pid" ]
    then
        echo $1: no such process.
        exit
    fi

    echo Dumping threads in process $pid
    echo
    ps $pid
    echo

    dbx -a $pid >/tmp/$$.th1 2> /dev/null <<EOF
print "\n------------- BEGIN: List of Process Threads -----------------------\n"
th
print "\n------------- END: List of Process Threads -------------------------"
print "\n------------- BEGIN: Condition Variables in Wait State -------------\n"
condition wait
print "\n------------- END: Condition Variables in Wait State ---------------"
print "\n------------- BEGIN: Mutexes in Lock State -------------------------\n"
mutex lock
print "\n------------- END: Mutexes in Lock State ---------------------------"
print "\n------------- BEGIN: List of Thread Stacks -------------------------\n"
detach
EOF

else
    source="core"
    # convert name to number

    CMD=$1
    CORE=$2

    if [ ! -f $CMD ]
    then echo "ERROR: no such process($CMD)."
        exit
    fi

    if [ ! -r "$CORE" ]
    then echo "ERROR: no core file($CORE) found."
        exit
    fi

    echo "Dumping threads for $CMD from $CORE\n"

    dbx $CMD $CORE >/tmp/$$.th1 2> /dev/null <<EOF
print "\n------------- BEGIN: List of Process Threads -----------------------\n"
th
print "\n------------- END: List of Process Threads -------------------------"
print "\n------------- BEGIN: Condition Variables in Wait State -------------\n"
condition wait
print "\n------------- END: Condition Variables in Wait State ---------------"
print "\n------------- BEGIN: Mutexes in Lock State -------------------------\n"
mutex lock
print "\n------------- END: Mutexes in Lock State ---------------------------"
print "\n------------- BEGIN: List of Thread Stacks -------------------------\n"
q
EOF
fi

awk '/^.\$t/ {
              gsub (">", "", $1)
              print "th current " substr($1,3)
              printf "th current\nt\n\n"
             }
     END {
          print "detach"
         }' /tmp/$$.th1 >>/tmp/$$.th2


if [ "$source" = "process" ]
then
    dbx -a $pid </tmp/$$.th2 >/tmp/$$.th3 2> /dev/null 
else
    dbx $CMD $CORE < /tmp/$$.th2 > /tmp/$$.th3 2> /dev/null
fi

if [ "$output" = "short" ]
then
print "\n------------- BEGIN: List of Thread Stacks -------------------------\n"
    awk '/thread  state-k/ {print thread "      " st}
	{
          i=split($1,fc,"(")
          if (st=="") st = fc[1]
          else st = fc[1] "->" st
         }
         /^>/ {
               gsub (">", "", $1)
               thread = substr($1,3)
               st=""
              }
	 /^0x/ {st=""}
         END {print thread "      " st}' /tmp/$$.th3
else
    cat /tmp/$$.th1
    awk '/thread  state/ {print ""} {print}' /tmp/$$.th3
 
fi
print "\n------------- END: List of Thread Stacks ---------------------------"
rm /tmp/$$.th*
